@@ -14,6 +14,11 @@
 #include "session.h"
 #include "utils.h"
 #include "waylandintegration.h"
+#include <KLocalizedString>
+#include <KNotification>
+#include <QGuiApplication>
+#include <QRegion>
+#include <QScreen>
 
 RemoteDesktopPortal::RemoteDesktopPortal(QObject *parent)
     : QDBusAbstractAdaptor(parent)
@@ -43,10 +48,6 @@ uint RemoteDesktopPortal::CreateSession(const QDBusObjectPath &handle,
         return 2;
     }
 
-    connect(session, &Session::closed, []() {
-        WaylandIntegration::stopAllStreaming();
-    });
-
     if (!WaylandIntegration::isStreamingAvailable()) {
         qCWarning(XdgDesktopPortalKdeRemoteDesktop) << "zkde_screencast_unstable_v1 does not seem to be available";
         return 2;
@@ -68,9 +69,10 @@ uint RemoteDesktopPortal::SelectDevices(const QDBusObjectPath &handle,
     qCDebug(XdgDesktopPortalKdeRemoteDesktop) << "    app_id: " << app_id;
     qCDebug(XdgDesktopPortalKdeRemoteDesktop) << "    options: " << options;
 
-    RemoteDesktopPortal::DeviceTypes types = RemoteDesktopPortal::None;
-    if (options.contains(QStringLiteral("types"))) {
-        types = static_cast<RemoteDesktopPortal::DeviceTypes>(options.value(QStringLiteral("types")).toUInt());
+    const auto types = static_cast<RemoteDesktopPortal::DeviceTypes>(options.value(QStringLiteral("types")).toUInt());
+    if (types == None) {
+        qCWarning(XdgDesktopPortalKdeRemoteDesktop) << "There are no devices to remotely control";
+        return 2;
     }
 
     RemoteDesktopSession *session = qobject_cast<RemoteDesktopSession *>(Session::getSession(session_handle.path()));
@@ -80,9 +82,6 @@ uint RemoteDesktopPortal::SelectDevices(const QDBusObjectPath &handle,
         return 2;
     }
 
-    if (options.contains(QStringLiteral("types"))) {
-        types = (DeviceTypes)(options.value(QStringLiteral("types")).toUInt());
-    }
     session->setDeviceTypes(types);
 
     return 0;
@@ -109,47 +108,58 @@ uint RemoteDesktopPortal::Start(const QDBusObjectPath &handle,
         return 2;
     }
 
-    // TODO check whether we got some outputs?
     if (WaylandIntegration::screens().isEmpty()) {
         qCWarning(XdgDesktopPortalKdeRemoteDesktop) << "Failed to show dialog as there is no screen to select";
         return 2;
     }
 
-    QScopedPointer<RemoteDesktopDialog, QScopedPointerDeleteLater> remoteDesktopDialog(
-        new RemoteDesktopDialog(app_id, session->deviceTypes(), session->screenSharingEnabled(), session->multipleSources()));
-    Utils::setParentWindow(remoteDesktopDialog->windowHandle(), parent_window);
-    Request::makeClosableDialogRequest(handle, remoteDesktopDialog.get());
+    if (app_id.isEmpty()) {
+        // non-sandboxed local applications are generally able to take over the system already without much hassle.
+        // Instead of making users interact with the dialog (which they probably can't because this is for remote access), just
+        // show a notification so they are aware of it.
+        auto notification = new KNotification(QStringLiteral("remotedesktopstarted"), KNotification::CloseOnTimeout);
+        notification->setTitle(i18nc("title of notification about input systems taken over", "Remote control session started"));
+        notification->setText(RemoteDesktopDialog::buildDescription(app_id, session->deviceTypes(), session->screenSharingEnabled()));
+        notification->setIconName(QStringLiteral("krfb"));
+        notification->sendEvent();
+    } else {
+        QScopedPointer<RemoteDesktopDialog, QScopedPointerDeleteLater> remoteDesktopDialog(
+            new RemoteDesktopDialog(app_id, session->deviceTypes(), session->screenSharingEnabled()));
+        Utils::setParentWindow(remoteDesktopDialog->windowHandle(), parent_window);
+        Request::makeClosableDialogRequest(handle, remoteDesktopDialog.get());
+        connect(session, &Session::closed, remoteDesktopDialog.data(), &RemoteDesktopDialog::reject);
 
-    connect(session, &Session::closed, remoteDesktopDialog.data(), &RemoteDesktopDialog::reject);
+        if (!remoteDesktopDialog->exec()) {
+            return 1;
+        }
+    }
 
-    if (remoteDesktopDialog->exec()) {
-        if (session->screenSharingEnabled()) {
-            WaylandIntegration::Streams streams;
-            const auto outputs = remoteDesktopDialog->selectedOutputs();
-            if (outputs.isEmpty()) {
-                return 2;
-            }
+    if (session->screenSharingEnabled()) {
+        WaylandIntegration::Streams streams;
+        if (session->multipleSources() || WaylandIntegration::screens().count() == 1) {
+            const auto outputs = WaylandIntegration::screens().values();
             for (const auto &output : outputs) {
-                auto stream = WaylandIntegration::startStreamingOutput(output.waylandOutputName(), Screencasting::Hidden);
+                auto stream = WaylandIntegration::startStreamingOutput(output.waylandOutputName(), Screencasting::Metadata);
                 if (!stream.isValid()) {
                     return 2;
                 }
                 streams << stream;
             }
-
-            results.insert(QStringLiteral("streams"), QVariant::fromValue<WaylandIntegration::Streams>(streams));
         } else {
-            qCWarning(XdgDesktopPortalKdeRemoteDesktop()) << "Only stream input";
+            streams << WaylandIntegration::startStreamingWorkspace(Screencasting::Metadata);
         }
-        WaylandIntegration::startStreamingInput();
-        WaylandIntegration::authenticate();
 
-        results.insert(QStringLiteral("devices"), QVariant::fromValue<uint>(remoteDesktopDialog->deviceTypes()));
-
-        return 0;
+        session->setStreams(streams);
+        results.insert(QStringLiteral("streams"), QVariant::fromValue<WaylandIntegration::Streams>(streams));
+    } else {
+        qCWarning(XdgDesktopPortalKdeRemoteDesktop()) << "Only stream input";
+        session->refreshDescription();
     }
+    session->acquireStreamingInput();
 
-    return 1;
+    results.insert(QStringLiteral("devices"), QVariant::fromValue<uint>(session->deviceTypes()));
+
+    return 0;
 }
 
 void RemoteDesktopPortal::NotifyPointerMotion(const QDBusObjectPath &session_handle, const QVariantMap &options, double dx, double dy)
@@ -218,6 +228,15 @@ void RemoteDesktopPortal::NotifyPointerAxis(const QDBusObjectPath &session_handl
     qCDebug(XdgDesktopPortalKdeRemoteDesktop) << "    options: " << options;
     qCDebug(XdgDesktopPortalKdeRemoteDesktop) << "    dx: " << dx;
     qCDebug(XdgDesktopPortalKdeRemoteDesktop) << "    dy: " << dy;
+
+    RemoteDesktopSession *session = qobject_cast<RemoteDesktopSession *>(Session::getSession(session_handle.path()));
+
+    if (!session) {
+        qCWarning(XdgDesktopPortalKdeRemoteDesktop) << "Tried to call NotifyKeyboardKeysym on non-existing session " << session_handle.path();
+        return;
+    }
+
+    WaylandIntegration::requestPointerAxis(dx, dy);
 }
 
 void RemoteDesktopPortal::NotifyPointerAxisDiscrete(const QDBusObjectPath &session_handle, const QVariantMap &options, uint axis, int steps)
@@ -240,10 +259,16 @@ void RemoteDesktopPortal::NotifyPointerAxisDiscrete(const QDBusObjectPath &sessi
 
 void RemoteDesktopPortal::NotifyKeyboardKeysym(const QDBusObjectPath &session_handle, const QVariantMap &options, int keysym, uint state)
 {
-    Q_UNUSED(session_handle)
     Q_UNUSED(options)
-    Q_UNUSED(keysym)
-    Q_UNUSED(state)
+
+    RemoteDesktopSession *session = qobject_cast<RemoteDesktopSession *>(Session::getSession(session_handle.path()));
+
+    if (!session) {
+        qCWarning(XdgDesktopPortalKdeRemoteDesktop) << "Tried to call NotifyKeyboardKeysym on non-existing session " << session_handle.path();
+        return;
+    }
+
+    WaylandIntegration::requestKeyboardKeysym(keysym, state != 0);
 }
 
 void RemoteDesktopPortal::NotifyKeyboardKeycode(const QDBusObjectPath &session_handle, const QVariantMap &options, int keycode, uint state)
